#!/usr/bin/perl -w

###########################################################################
# This script takes a SAT formula, translates and feeds it into Z3,
# and interprets the result.
# A formula can be given on standard input, in a file, or on the command line.
# Invoke the script with -h for help.
#
# Syntax for the formula:
#   Conjunction: one of a*b, a /\ b, a&b
#   Disjunction: one of a+b, a \/ b, a|b
#   Negation: one of -b, !b
#   Implication: one of a->b, a=>b, a<b
#   Equivalence: one of a<->b, a<=>b, a=b
#
# When using the command line, think of putting the formula in quotes
# (and escape ! with \!).
###########################################################################

use IPC::Open2;

$PROG = "satcheck";

$optdisp = 0;
$fn = shift @ARGV;
if (defined($fn) && $fn eq "-d") {
	$fn = shift @ARGV;
	$optdisp = 1;
}
if (!defined($fn)) {
	print STDERR "$PROG -- (invoke with -h for help)\n";
	print STDERR "reading formula from standard input\n";
	@lines = <>;
} elsif ($fn eq "-f") {
	$fn = shift @ARGV;
	print STDERR "reading formula from file $fn\n";
	die "cannot read file $fn" unless -r $fn;
	open FH,$fn;
	@lines = <FH>;
	close FH;
} elsif ($fn eq "-c") {
	print STDERR "reading formula from command line\n";
	@lines = (shift @ARGV);
} else {
	print "\nusage: $PROG -- satisfiability test for SAT\n",
		"  $PROG [-d] -f <filename>: read formula from file\n",
		"  $PROG [-d] -c <formula>: read formula from command line\n",
		"  $PROG [-d]: read formula from standard input line\n",
		"   -d = only show SMT2 output, no solving\n\n";
	exit 2;
}

open2(*Reader,*Writer,"./f2z3");
print Writer @lines;
close Writer;
@zin = <Reader>;
close Reader;

if ($optdisp) {
	print @zin;
	exit 0;
}

open2(*Reader,*Writer,"z3 -smt2 -in");
print Writer @zin;
close Writer;
while (<Reader>) {
	tr/\015\012//d;
	if (/^unsat$/) {
		print "The formula is UNSATISFIABLE.\n";
		exit 1;
	}
	if (/^sat$/) {
		print "The formula is SATISFIABLE as follows:\n";
		next;
	}
	$i = $1 if /define-fun (.*?) \(/;
	if (/true/) { printf "$i = true\n"; }
	if (/false/) { printf "$i = false\n"; }
}

exit 0;
